function [p,c,rank] = DSFLocUknPropSpdStnySrc_SDR(s,s_dot,f,Q,f0)
% function [p,c,rank] = DSFLocUknPropSpdStnySrc_SDR(s,s_dot,f,Q,f0)
%
% SDR for the stationary source localization 
% This function is the SDR method in the reference.
%
% Input:
%   s:        (2 x N), the true value of sensor positions, N = number of sensors
%   s_dot:    (2 x N), the true value of sensor velocities
%   f:        (1 x N), the frequency measurements
%   Q:        (N x N), the covariance matrix of measurement noise
%   f0:       (1 x 1), the carrier frequency
%
% Output:
%   p:       (2 x 1), the estimate of source position
%   c:       (1 x 1), the estimate of signal propagation speed
%   rank:     (1 x 1), the rank of SDP solution
%
% Reference:
% D. Lin, G. Wang, K. C. Ho, and L. Huang, "Source localization by frequency
% measurements in unknown signal propagation speed environments," IEEE
% Trans. Aerosp. Electron. Syst., vol. 59, no. 4, pp. 3953-3970, Aug. 2023.
%
% Danyan Lin, Gang Wang, K. C. Ho	Nov. 1, 2023
%
%       Copyright (C) 2023
%       the Faculty of Electrical Engineering and Computer Science
%       Ningbo University
%       Ningbo 315211, China
%       wanggang@nbu.edu.cn
%

warning('off');

N = length(f);

b = [];
d = [];

for i=1:N  
    d(i) = -(f0-f(i))/f0;
    A(i,:) = [2*s(:,i)'*s_dot(:,i)*s_dot(:,i)',d(i)^2*norm(s(:,i))^2,-2*d(i)^2*s(:,i)',-s_dot(1,i)^2,-2*s_dot(1,i)*s_dot(2,i),-s_dot(2,i)^2,d(i)^2];
    b = [b;(s(:,i)'*s_dot(:,i))^2];    
end

W = inv(Q);

for i=1:3
    B = [];
    W = W/trace(W);
    cvx_clear
    cvx_begin sdp
    cvx_solver sdpt3
    cvx_quiet(1)
    cvx_precision best
    variable Y(9,9) symmetric      
    variable y(9,1)

    minimize ((trace(A'*W*A*Y)-2*b'*W*A*y+b'*W*b)*1e11)

    subject to
    [Y y;y' 1]>=0;
    y(4) == Y(1,3);
    y(5) == Y(2,3);
    y(6) == Y(1,1);
    y(7) == Y(1,2);
    y(8) == Y(2,2);
    y(9) == Y(3,6)+Y(3,8);
    
    Y(1,4) == Y(3,6);
    Y(1,5) == Y(3,7);
    Y(1,5) == Y(2,4);
    Y(1,7) == Y(2,6);
    Y(1,8) == Y(2,7);
    Y(1,9) == Y(4,6)+Y(5,7);
    Y(2,5) == Y(3,8);
    Y(2,9) == Y(4,7)+Y(5,8);
    Y(3,9) == Y(4,4)+Y(5,5);
    Y(4,7) == Y(5,6);
    Y(4,8) == Y(5,7);
    Y(6,8) == Y(7,7);
    Y(1,1)+2*Y(1,2)+Y(2,2) == y(6)+2*y(7)+y(8);

    cvx_end
    
    p = y(1:2);
    c = sqrt(y(3));
    
    for j=1:N
        B = blkdiag(B,2*d(j)*c^2/f0*norm(p-s(:,j))^2);
    end
    W = inv(B'*Q*B);
    
    rank=0;
    lambda=[];
    lambda=eig(Y);
    if lambda(9)/lambda(8)>=1e6
        rank=1;
    end

end

rank=0;
lambda=[];
lambda=eig(Y);
if lambda(9)/lambda(8)>=1e7
    rank=1;
end
 
p = y(1:2);
c = sqrt(y(3));
end

